# Copyright (C) 2023 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2024, Advanced Micro Devices, Inc."

import os
import psutil
import json
import re
from pathlib import Path
import subprocess


class Stats(object):
    hwmon_list = []
    def __init__(self, hwmons):
        self.hwmon_list = hwmons


    def getCpuUtilization(self):
        cpu_percent_per_core = psutil.cpu_percent(interval=1, percpu=True)
        
        cpu_utilization = {"CPU Utilization": {}}
        for i, cpu_percent in enumerate(cpu_percent_per_core):
            cpu_utilization["CPU Utilization"][f"CPU Core {i + 1}"] = cpu_percent

        return cpu_utilization

    def getCpuFrequencies(self):
        cpu_frequencies = {"CPU Frequencies": {}}
        for i, freq in enumerate(psutil.cpu_freq(percpu=True)):
            cpu_frequencies["CPU Frequencies"][f"CPU Core {i + 1}"] = {
                "Current Frequency": freq.current,
                "Max Frequency": freq.max,
                "Min Frequency": freq.min
            }

        return cpu_frequencies

    def getMemoryUsage(self):
        memory = psutil.virtual_memory()
        used_memory = memory.used
        free_memory = memory.free
        total_memory = memory.total

        with open('/proc/meminfo') as meminfo_file:
            meminfo_data = meminfo_file.read()

        cma_total_line = next(line for line in meminfo_data.split('\n') if 'CmaTotal' in line)
        cma_free_line = next(line for line in meminfo_data.split('\n') if 'CmaFree' in line)

        cma_total = int(cma_total_line.split()[1])
        cma_free = int(cma_free_line.split()[1])

        swap = psutil.swap_memory()
        swap_free = swap.free
        swap_total = swap.total
        swap_percent = swap.percent

        memory_usage = {
            "Memory Usage": {
                "Used Memory": used_memory / (1024 ** 2),
                "Free Memory": free_memory / (1024 ** 2),
                "Total Memory": total_memory / (1024 ** 2)
            },

            "CMA Mem Utilization": {
                "Cma Total" : cma_total,
                "Cma Free" : cma_free
            },
            
            "Swap Mem Utilization": {
                "Swap Free": swap_free / (1024 ** 2),
                "Swap Total": swap_total / (1024 ** 2),
                "Swap Usage": swap_percent
            }
        }

        return memory_usage

    def getVoltages(self):
        voltages = {"Voltage Info": {
            "PS Sysmon": {},
            "PL Sysmon": {},
            "AMS CTRL": {}
        }}
        voltages
        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage9_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage9_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCC PS LPD": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage10_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage10_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCC PS FPD": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage11_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage11_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"PS AUX": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage12_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage12_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"DDR IO VCC": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage13_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage13_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"PS IO BANK 503": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage14_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage14_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"PS IO BANK 500": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage15_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage15_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCCO_PSIO1": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage16_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage16_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCCO_PSIO2": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage17_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage17_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCC_PS_GTR": val})
        
        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage18_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage18_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VTT_PS_GTR": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage19_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage19_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PS Sysmon"].update({"VCC_PSADC": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage21_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage21_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCCINT": val})
        
        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage22_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage22_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCCAUX": val})
        
        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage23_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage23_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"ADC Ref P+": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage24_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage24_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"ADC Ref N-": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage25_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage25_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCCBRAM": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage26_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage26_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCC_PSINTLP": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage27_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage27_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCC_PSINTFP": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage28_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage28_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"PS AUX": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage29_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage29_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["PL Sysmon"].update({"VCCAMS": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage0_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage0_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCC_PSPLL": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage1_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage1_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCC_PSBATT": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage2_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage2_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCCINT": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage3_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage3_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCCBRAM": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage4_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage4_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCCAUX": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage5_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage5_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCC_PSDDR_PLL": val})

        ret1, raw = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage6_raw')
        ret2, scale = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_voltage6_scale')
        if  (ret1 | ret2) == 0:
            val = round((int(raw) * float(scale))/1000, 3)
            voltages["Voltage Info"]["AMS CTRL"].update({"VCC_PSINTFP_DDR": val})

        return voltages

    def getTemperatures(self):
        temps = {"Temperature Info": {}}
        ret1, pl_temp = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_temp20_input')
        ret2, lpd_temp = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_temp7_input')
        ret3, fpd_temp = subprocess.getstatusoutput('cat /sys/bus/iio/devices/iio\:device0/in_temp8_input')
        if (ret1 | ret2 | ret2) == 0:
            temps["Temperature Info"]["FPD Temp"] = float(int(fpd_temp)/1000)
            temps["Temperature Info"]["LPD Temp"] = float(int(lpd_temp)/1000)
            temps["Temperature Info"]["PL Temp"] = float(int(pl_temp)/1000)
        return temps

    def getLastStatus(self):
        status = {}
        status.update(self.getCpuUtilization())
        status.update(self.getCpuFrequencies())
        status.update(self.getMemoryUsage())
        status.update(self.getTemperatures())
        status.update(self.getVoltages())

        return status


# Copyright (C) 2023 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023, Advanced Micro Devices, Inc."

import Pyro4
import logging
import base64
from enum import Enum
from periphery import I2C

logging.basicConfig(level=logging.ERROR)


class DeviceType(Enum):
    INA226 = 1
    INA230 = 2
    SYSMON = 99
    #Possible other Power Monitor IC definitions here.
class I2C_Client(object):

    def __init__(self, device_no, device_addr):
        logging.info("Inside I2C_Client Constructor")
        self.addr = int(device_addr, 0)
        self.device_no = device_no
        self.device = None


        logging.debug(
            "I2C_Client Initialize(" + str(self.device_no) +  ")"
        )
        try:
            self.device = I2C(self.device_no)
        except IOError:
            logging.error("I2C_Client Initialize failed.")
        

    def Initialize(self):
        """
        I2C driver one time initialisation.

        :param device_id: id of the opened device
        :return: ret_code
        """
        ret = True
        logging.debug(
            "I2C_Client Initialize(" + str(self.device_no) +  ")"
        )
        #self.device_type = d_type
        #try:
        #    self.device = I2C("/dev/i2c-" + str(number))
        #    ret = True
        #except IOError:
        #    logging.error("I2C_Client Initialize failed.")
        return ret
    
    def ReadRegister(self, reg, length):
        """
        I2C driver read a register

        :param reg: register to be read
        :param length: length to be read
        :return: ret_code, value of register
        """
        ret = False
        msgs = None
        data  = bytearray(length)
        try:
            msgs = [I2C.Message([0x0, 0x0]), I2C.Message(data, read=True)]
            self.device.transfer(self.addr, msgs)
            #logging.debug("{0}: 0x{1:02x}{2:02x}".format(self.addr, msgs[1].data[0], msgs[1].data[1]))
            ret = True
        except IOError:   
            logging.error("I2C_Client  failed.")
        return ret, msgs[1].data

    def WriteRegister(self, reg, val):
        """
        I2C driver write a register

        :param reg: register to be written
        :param val: value to be written
        :return: ret_code
        """
        ret = False
        data = None
        try:
            data = bytearray(val.to_bytes(2, "big"))
            data.insert(0, reg)
            msgs = [I2C.Message(data)]
            self.device.transfer(self.addr, msgs)
            ret = True
        except IOError:   
            logging.error("I2C_Client WriteINA226Reg failed.")
        return ret

    def Release(self):
        """
        I2C driver Release instance
        :return: none
        """
        self.device.close()

    def __del__(self):
        logging.info("Inside I2C Destructor")



# Copyright (C) 2023 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023, Advanced Micro Devices, Inc."

import os 
import platform
import sys
import json
sys.path.append('../../utils')
sys.path.append('../../raft_services')
sys.path.append('../../raft_services/power_management')
import Pyro4
from pathlib import Path
from pm import PM
from pmic_i2c import I2C_Client
from utils import get_ip_and_port

RAFT_DIR = '/usr/share/raft/'
SYS_CTL_APP_DIR = '/usr/share/system-controller-app/'

IPADDR, PORT = get_ip_and_port()
if len(IPADDR) == 0:
    print("CRITITCAL ERROR: Unable to Run Pyro Server.\n"
          "No eth0 network interface present.\n"
          "Please pass the ipaddress to __init__.py.\n"
          "Usage:python3 __init__.py <ipaddress> <port>\n")
    sys.exit()


def exit_program():
    print("CRITITCAL ERROR: Unable to Run Pyro Server.\n")
    sys.exit(1)

def is_valid_json_file(file_path):
    if not os.path.isfile(file_path):
        print("ERROR:root:Board Identification Json is not in Path : {file_path}")
        return False
    try:
        with open(file_path, 'r') as file:
            data = json.load(file)
            return True
    except json.JSONDecodeError:
        print("ERROR:root:{file} Board Identification is not a JSON")
        return False

class board_eeprom:
    Name = ""
    I2C_Bus = ""
    I2C_Addr = ""

onboard = board_eeprom()

def parse_json_file(json_file):
    with open(json_file, 'r') as f:
        json_data = json.load(f)
    return json_data

def get_eeprom_data():
    result = False
    i2c = None
    onboard.Name = "Common"
    onboard.I2C_Bus = "/dev/i2c-1"
    onboard.I2C_Addr = "0x54"
    i2c = I2C_Client(onboard.I2C_Bus, onboard.I2C_Addr) # Common_OnBoard_EEPROM
    if i2c.device is not None:
        result, eeprom_data = i2c.ReadRegister(0x00, 256)
    if result is False:
        #i2c.Release()
        onboard.Name = "Legacy"
        onboard.I2C_Bus = "/dev/i2c-11"
        onboard.I2C_Addr = "0x54"
        i2c = I2C_Client(onboard.I2C_Bus, onboard.I2C_Addr)
        if i2c.device is not None:
            result, eeprom_data = i2c.ReadRegister(0x00, 256)
        if result is False:
            print("ERROR:root:Board Identification Eeprom Failed.")
            exit_program()
        i2c.Release()
    return eeprom_data

def get_product_name():
    eeprom_data = get_eeprom_data()
    offset = 0x15
    length = int.from_bytes(eeprom_data[offset:offset+1], "big") & 0x3f
    name = eeprom_data[offset+1:(offset+1 + length)].decode("utf-8").strip('\x00')
    return name

if(len(sys.argv) > 2):

    product_name = get_product_name()
    #json_file = os.path.join(RAFT_DIR, 'xserver/raft_services/power_management/boards', '.'.join((product_name, 'json')))
    json_file = os.path.join(SYS_CTL_APP_DIR, 'board', '.'.join((product_name, 'json')))

    if is_valid_json_file(json_file):
        json_data = parse_json_file(json_file)
        board_name = os.path.splitext(os.path.basename(json_file))
    else:
        exit_program()

PM = Pyro4.expose(PM)

Pyro4.Daemon.serveSimple(
    {
        PM(json_data, board_name[0], 'INA226', onboard): "PM",
    },
    host=IPADDR,
    port=PORT,
    ns=False,
    verbose=True,
)




# Copyright (C) 2023 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023, Advanced Micro Devices, Inc."

import sys
import json
import argparse
sys.path.append('/usr/share/raft/xclient/raft_services')
from pm_client import PM_Client


def main():
    client = PM_Client()
    
    parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
    parser.add_argument("-c", "--command", required=True, choices=["boardinfo", "domains", "powervalue", "rails", "raildetail", "railvalue", "loglevel", "pstemp", "sysmon", "stats"],
                            help="Specify the command (boardinfo, domains, rails or railvalue)")

    args, remaining_args = parser.parse_known_args()

    if args.command:
        if args.command == "rails":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)

            if target_args.target:
                print(json.dumps(client.GetRailsOfDomain(target_args.target)))
        
        elif args.command == "railvalue":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)

            if target_args.target:
                print(json.dumps(client.GetValueOfRail(target_args.target)))
        elif args.command == "raildetail":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=True, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)

            if target_args.target:
                print(json.dumps(client.GetRailDetails(target_args.target)))

        elif args.command == "boardinfo":
            print(json.dumps(client.GetBoardInfo()))
        
        elif args.command == "domains":
            print(json.dumps(client.GetPowerDomains()))

        elif args.command == "powervalue":
            print(json.dumps(client.GetPowersAll()))

        elif args.command == "loglevel":
            target_parser = argparse.ArgumentParser(description="Power Management Client Module CLI")
            target_parser.add_argument("-t", "--target", required=False, help="Specify the target")
            target_args, target_remining_args = target_parser.parse_known_args(remaining_args)

            if target_args.target is None:
                print(json.dumps(client.logger.level))
            else:
                if target_args.target:
                    print(f"Target: {target_args.target}")
        
        elif args.command == "sysmon":
            print(json.dumps(client.GetSysmonTemperatures()))
            
        elif args.command == "pstemp":
            print(json.dumps(client.GetPSTemperature()))

        elif args.command == "stats":
            print(json.dumps(client.GetSystemStats()))
        
        else:
            args = parser.parse_args(remaining_args)

if __name__ == "__main__":
    # call main
    main()




# Copyright (C) 2023 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023, Advanced Micro Devices, Inc."

import sys
import json
import sys
def usage():
    # sys.argv[0] - usage python file name
    # sys.argv[1] - xclient/xpyro
    # sys.argv[2] - host/board
    # sys.argv[3] - ip_address
    # sys.argv[4] - port
    print(f"Usage: The application work in three modes xclient and xpyro\n"
          f"python3 {sys.argv[0]} xclient host/board ip_address port => Runs at host or board with xclient\n"
          f"python3 {sys.argv[0]} xpyro host/board ip_address port => Runs at host or board with xpyro\n"
          f"Example: python3 {sys.argv[0]} xclient host 127.0.0.1 9090")

if (len(sys.argv) !=  5):
    usage()
    sys.exit()

# The 'xclient' option will run both in host and board
if (sys.argv[1] == 'xclient'):
    # For 'xclient' option the path of the xclient is required
    # The relative path in the host is given for host case
    if (sys.argv[2] == 'host'):
        sys.path.append('../../../xclient/raft_services')
    # The location of xclient in the filesystem is given for the board case
    elif (sys.argv[2] == 'board'):
        sys.path.append('/usr/share/raft/xclient/raft_services')
    else:
        usage()
        sys.exit()
    import pm_client
    handle = pm_client.pm
    ip_address = sys.argv[3]
    port = sys.argv[4]

# The 'xpyro' option will run both in host and board
elif (sys.argv[1] == 'xpyro'):
    ip_address = sys.argv[3]
    port = sys.argv[4]
    import Pyro4
    # Prepare the uri needed for pyro communication
    uri = f"PYRO:PM@{ip_address}:{port}"
    handle = Pyro4.Proxy(uri)
else:
    usage()
    sys.exit()

if (sys.argv[1] == 'xclient'):
    #SetIpAndPort
    handle.SetIpAndPort(ip_address, port)
    #GetPythonLogLevels
    PythonLogLevels = handle.GetPythonLogLevels()
    #SetClientLogLevel
    handle.SetClientLogLevel(PythonLogLevels["DEBUG"])


print(f'######## BOARD INFO ########')
#Description:
#   Gets Board info.
#Input Arguments:
#
#Return:
#   ret:BoardInfo in Json format.
ret = handle.GetBoardInfo()
print(json.dumps(ret, indent=2))
print(f'#################################')

print(f'######## LIST OF DOMAINS ########')

#Description:
#   Gets Power domains list.
#Input Arguments:
#   None
#Return:
#   ret: Power Domains in Json format.
ret = handle.GetPowerDomains()
print(json.dumps(ret, indent=2))
print(f'#################################')


print(f'######## LIST OF "PLD" RAILS ########')
domain_name = "FPD"

#Description:
#   Gets Rail list of asked domain
#Input Arguments:
#   domain_name: Domain Name
#Return:
#   ret: List of rails for asked domain.
ret = handle.GetRailsOfDomain(domain_name)
print(json.dumps(ret, indent=2))
print(f'#####################################')

print(f'######## DETAILS OF "VCCINT_PSFP" RAIL ########')
rail_name = "VCCINT_PSFP"

#Description:
#   Gets rail details.
#Input Arguments:
#   rail_name: Rail Name
#Return:
#   ret: List of rails for asked domain.
ret = handle.GetRailDetails(rail_name)
print(json.dumps(ret, indent=2))
print(f'###########################################')

print(f'######## VALUES OF "VCCINT_PSFP" RAIL ########')
rail_name = "VCCINT_PSFP"

#Description:
#   Gets power/sensor values of a rail.
#Input Arguments:
#   rail_name: Rail Name
#Return:
#   ret: List of rails for asked domain.
ret = handle.GetValueOfRail(rail_name)
print(json.dumps(ret, indent=2))
print(f'##########################################')

print(f'######## VALUES OF "PLD" DOMAIN ########')
domain_name = "PLD"

#Description:
#   Gets power/sensor values of a domain.
#Input Arguments:
#   rail_name: Domain Name
#Return:
#   ret: List of rails for asked domain.
ret = handle.GetValueOfDomain(domain_name)
print(json.dumps(ret, indent=2))
print(f'########################################')

print(f'######## GET ALL DOMAINS POWER VALUES AT ONCE ##########')
#Description:
#   Gets domains and total power values of the board.
#Input Arguments:
#   None
#Return:
#   ret: List of power values of the board.
ret = handle.GetPowersAll()
print(json.dumps(ret, indent=2))
print(f'########################################')

print(f'######## GET ALL DOMAINS VALUES AT ONCE ##########')
#Description:
#   Gets power/sensor values of the board.
#Input Arguments:
#   None
#Return:
#   ret: List of rails for asked domain.
ret = handle.GetValuesAll()
print(json.dumps(ret, indent=2))
print(f'########################################')


print(f'######## GET SYSMON TEMPERATURE VALUES ##########')
#Description:
#   Gets sysmon temperature values of Versal.
#Input Arguments:
#   None
#Return:
#   ret: Sysmon Temperature values
ret = handle.GetSysmonTemperatures()
print(json.dumps(ret, indent=2))
print(f'########################################')


print(f'######## GET PS TEMPERATURE VALUE ##########')
#Description:
#   Gets PS temperature value of Versal.
#Input Arguments:
#   None
#Return:
#   ret: PS Temperature values.
ret = handle.GetPSTemperature()
print(json.dumps(ret, indent=2))
print(f'########################################')


print(f'######## GET SYSTEM STATS ##########')
#Description:
#   Gets PS temperature value of Versal.
#Input Arguments:
#   None
#Return:
#   ret: PS Temperature values.
ret = handle.GetSystemStats()
print(json.dumps(ret, indent=2))
print(f'########################################')







# Copyright (C) 2021-2022 Xilinx, Inc.  All rights reserved.
# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All Rights Reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Hugh Maguire"
__copyright__ = "Copyright 2021, Xilinx"

import sys
sys.path.append('../../utils')
sys.path.append('../../xcffi/drv_api/rfdc')
import Pyro4
from rfdc_server import RFDC
from utils import get_ip_and_port

IPADDR, PORT = get_ip_and_port()
if len(IPADDR) == 0:
    print("CRITITCAL ERROR: Unable to Run Pyro Server.\n"
          "No eth0 network interface present.\n"
          "Please pass the ipaddress to __init__.py.\n"
          "Usage:python3 __init__.py <ipaddress> <port>\n")
    sys.exit()

RFDC = Pyro4.expose(RFDC)

Pyro4.Daemon.serveSimple(
    {
        RFDC: "RFDC"
    },
    host=IPADDR,
    port=PORT,
    ns=False,
    verbose=True,
)



# Copyright (C) 2023 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023, Advanced Micro Devices, Inc."

import os
import platform
import sys
import json

RAFT_DIR = '/usr/share/raft/'

sys.path.append(RAFT_DIR + 'xserver/utils')
#sys.path.append(os.environ['RAFT_DIR'] + 'xserver/utils')
import logging
from pmic import PMIC
from pmic import Rails
from pmic import Domain
from pmic import DeviceType
from sysmon import Sysmon_Device_Type
from sysmon import Sysmon
from stats import Stats
from utils import get_python_log_levels

        
class PM(object):
    device_id = 0
    logger = None
    domains = []
    rails = []
    pmic = None
    board_name = ""
    sysmons = []
    status = None
    voltages = []

    def __init__(self, json_data, board_name, rail_prefix, eeprom):
        self.logger = self.GetLogger()
        self.boardeeprom = eeprom
        self.board_name = board_name
        board_data = json_data[board_name]
        power_domains_data = {}
        temperature_data = {}
        hwmon_data = {}

        if 'POWER DOMAIN' in board_data:
            power_domains_data = board_data['POWER DOMAIN']
        
        if 'VOLTAGE' in board_data:
            hwmon_data = board_data['VOLTAGE']
        
        if rail_prefix in board_data:
            rails_data = board_data[rail_prefix]
        
        for k, v in power_domains_data.items():
            temp_d = Domain(**v)
            for railname in temp_d.railnames:
                for k, v in rails_data.items():
                    if k == railname:
                        temp_r = Rails(**v)
                        temp_r._device_type = DeviceType.INA226 # fix this !!!
                        self.rails.append(temp_r)
                        temp_d.rails.append(temp_r)
            self.domains.append(temp_d)

        for k, v in hwmon_data.items():
            temp_r = Rails(v['Name'], v['I2C_Bus'], v['I2C_Address'], "", "", "")
            self.voltages.append(temp_r)
   
        if 'Temperature' in board_data:
            sensor_data = board_data['Temperature']['Sensor']
            temp_s = None
            if "versal-isa-0000" in sensor_data:
                try:
                    temp_s = Sysmon(None, None, Sysmon_Device_Type.sysfs)
                    if temp_s is not None:
                        self.sysmons.append(temp_s)
                except Exception as e:
                    print(e)
            
            if "i2c" in sensor_data:
                tokens = sensor_data.split("-")
                address = tokens[3]
                device_path = "/dev/i2c-" + tokens[2]
                try:
                    temp_s = Sysmon(address, device_path, Sysmon_Device_Type.iic)
                    if temp_s is not None:
                        self.sysmons.append(temp_s)
                except Exception as e:
                    print(e)
                
        try:
            self.status = Stats(self.voltages)
            if self.status is None:
                self.logger.error(f"PM: InitBoardInfo failed. ret = {ret}")
        except Exception as e:
            self.logger.error(f"PM: InitBoardInfo failed. ret = {ret}")
            print(e)
            exit_program()

        try:
            self.pmic = PMIC(self.domains)
        except Exception as e:
            print(e)
            exit_program()

        
        if self.pmic is None:
            self.logger.error(f"PM: InitBoardInfo failed. ret = {ret}")
        self.logger.info("Inside PM Constructor")

    @staticmethod
    def exit_program():
        print("CRITITCAL ERROR: Unable to Run Pyro Server.\n")
        sys.exit(1)
    
    @staticmethod
    def GetLogger():
        """
        Static method to get the logger for the class.
        Default loglevel is set inside this class

        :return: logger

        """
        log_level = logging.INFO
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            logger.handler_set = True
            logger.disabled = False
        return logger

    # Log level
    def GetPythonLogLevels(self):
        """
        Return the logging levels supported by logging library in python

        :param : None
        :return: Dictionary showing the log levels supported by logging library
        """
        return get_python_log_levels()

    def SetServerLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"PythonLogLevel = {PythonLogLevel}")
        LogLevelsDict = get_python_log_levels()
        if PythonLogLevel == LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def GetBoardInfo(self):
        """
        Gets Board's Info

        :param : None 
        :return: Board Info in json formatted
        """
        return self.pmic.BoardInfo(self.boardeeprom)
    
    def GetPSTemperature(self):
        """
        Gets SysCtl Temperature Info

        :param : None 
        :return: SysCtl Temperature Info in json formatted
        """
        return self.status.getTemperatures()
    
    def GetPowerDomains(self):
        """
        Gets list of Power Domains.

        :param : None
        :return: Domains in json formatted
        """
        powerdomains = {}
        powerdomains['POWER DOMAINS'] = []
        for d in self.domains:
            for k, v in d.__dict__.items():
                if k in ['name'] and v:
                    temp = {k: v}
                    powerdomains['POWER DOMAINS'].append(temp)
        return powerdomains

    def GetRailsOfDomain(self, domainname):
        """
        Gets list of Rails given domain name.

        :param domainname: string of a "domainname" 
        :return: Rails in json formatted
        """
        rails = {}
        rails[domainname] = []
        for d in self.domains:
            if d.name == domainname:
               for r in d.rails:
                for k, v in r.__dict__.items():
                    if k in ['name'] and v:
                        temp = {k: v}
                        rails[domainname].append(temp)
        return rails
    
    def GetRailDetails(self, railname):
        """
        Gets list of the rail's details given rail name.

        :param railname: string of a "railname" 
        :return: Details of the Rail in json formatted
        """
        details = {}
        for d in self.domains:
            for r in d.rails:
                if railname == r.name:
                    details[r.name] = {}
                    for k, v in r.__dict__.items():
                        if k not in ['name', 'sensor', '_sensor', '_device_type'] and v:
                                details[r.name][k] = v
        return details
    
    @staticmethod
    def GetRailValues(domain, rail):
        val = None
        for rail in domain.rails:
            val = self.GetValueOfRail(rail)

    
    def GetSensorValue(self, sensor, railname):
        value = {}
        v, i, p = self.pmic.GetSensorValues(sensor)
        value[railname] = {}
        value[railname]['Voltage'] = round(v, 4)
        value[railname]['Current'] = round(i, 4)
        value[railname]['Power'] = round(p, 4)
        return value

    def GetPowerValue(self, sensor, railname):
        value = {}
        v, i, p = self.pmic.GetSensorValues(sensor)
        value[railname] = {}
        value[railname]['Power'] = round(p, 4)
        return value

    def GetValueOfRail(self, railname):
        """
        Gets list of the rail's sensor values given rail name.

        :param railname: string of a "railname" 
        :return: Sensor values of the Rail in json formatted
        """
        data = None
        sensor = None
        for domain in self.domains:
            for rail in domain.rails:
                if rail.name == railname:
                    sensor = self.pmic.GetSensor(domain.name, rail.name)
        if sensor is not None: 
            data = self.GetSensorValue(sensor, railname)
        return data

    def GetValueOfDomain(self, domainname):
        """
        Gets the domain's all rail sensor values given domain name.

        :param : string of a "domainname" 
        :return: The domain's all rails sensor values of the Rail in json formatted
        """
        total_power = 0.0
        data = {}
        sensor = None
        data[domainname] = {}
        data[domainname]['Rails'] = []
        for domain in self.domains:
            if domain.name == domainname:
                for rail in domain.rails:
                    sensor = self.pmic.GetSensor(domain.name, rail.name)
                    if sensor is not None:
                        temp_v = self.GetSensorValue(sensor,rail.name)
                        data[domainname]['Rails'].append(temp_v)
                        total_power += temp_v[rail.name]['Power']
        data[domainname]['Total Power'] = round(total_power, 4)
        return data

    def GetPowerValueOfDomain(self, domainname):
        """
        Gets the domain's power value given domain name.

        :param : string of a "domainname" 
        :return: The domain's power value in json formatted
        """
        total_power = 0.0
        domain_power = 0.0
        data = {}
        data[domainname] = {}
        sensor = None
        for domain in self.domains:
            if domain.name == domainname:
                domain_power = 0.0
                for rail in domain.rails:
                    sensor = self.pmic.GetSensor(domain.name, rail.name)
                    if sensor is not None:
                        temp_p = self.GetPowerValue(sensor,rail.name)
                        domain_power += temp_p[rail.name]['Power']
                data[domainname]['Power'] = round(domain_power, 4)
        return data

    def GetPowersAll(self):
        """
        Gets the boards's all domain's and total power values

        :param : None 
        :return: The boards's all domain's and total power values
        """
        data = {}
        data[self.board_name] = {}
        data[self.board_name]['Power Domains'] = []
        total_power = 0.0
        for domain in self.domains:
            temp_p = self.GetPowerValueOfDomain(domain.name)
            total_power += temp_p[domain.name]['Power']
            data[self.board_name]['Power Domains'].append(temp_p)
        data[self.board_name]['Total Power'] = round(total_power, 4)
        return data

    def GetValuesAll(self):
        """
        Gets the boards's all domain's rails sensor values

        :param : None 
        :return: The board's all rails sensor values of the Rail in json formatted
        """
        data = {}
        data[self.board_name] = []
        for domain in self.domains:
            data[self.board_name].append(self.GetValueOfDomain(domain.name))
        return data
    

    def GetSysmonTemperatures(self):
        if self.sysmons is None:
            return None
        data = {}
        for s in self.sysmons:
            data = {}
            temperature, minimum, max_max, min_min = s.ReadSysmonTemperatures()
            data['TEMP'] = temperature
            data['MIN'] = minimum
            data['MAX_MAX'] = max_max
            data['MIN_MIN'] = min_min
        return data
        
    def GetSystemStats(self):
        data = {}
        data = self.status.getLastStatus()
        return data
    
    def __del__(self):
        self.logger.info("Inside PM Destructor")



# Copyright (C) 2023 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023, Advanced Micro Devices, Inc."

import os
import platform
import sys
import subprocess
import time


RAFT_DIR = '/usr/share/raft/'

sys.path.append(RAFT_DIR + 'xserver/utils')
import logging
import json
from enum import Enum
from pmic_i2c import I2C_Client
from pmic_ina226 import INA226
#from pmic_spi import spi

class Rails:
    def __init__(self, Name, I2C_Bus, I2C_Address, Shunt_Resistor, Maximum_Current, Phase_Multiplier):
        self.name = Name
        self.i2c_bus = I2C_Bus
        self.i2c_address = I2C_Address
        self.shunt_resistor = Shunt_Resistor
        self.maximum_current = Maximum_Current
        self.phase_multiplier = Phase_Multiplier
        self._device_type = 0
        self._sensor = None

    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k not in ['i2c_bus', 'i2c_address', 'shunt_resistor', 'maximum_current','phase_multiplier', '_device_type', '_sensor'] and v
        }
        return str(str_info)
    pass
class Domain:
    def __init__(self, Name, Rails):
        self.name = Name
        self.railnames = Rails
        self.rails = []
    
    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k not in ['railnames', 'rails'] and v
        }
        return str(str_info)

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, 
            sort_keys=True, indent=4)


class DeviceType(Enum):
    INA226 = 1
    INA230 = 2
    SYSMON = 99
    #Possible other Power Monitor IC definitions here.

class SensorData:
    def __init__(self, name, domain_name, device, Shunt_Resistor, Maximum_Current , Phase_Multiplier):
        self.name = name
        self.domain_name = domain_name
        self.device = device
        self.shunt_resistor = Shunt_Resistor
        self.maximum_current = Maximum_Current
        self.phase_multiplier = Phase_Multiplier
        self.vbus = 0.0
        self.current = 0.0
        self.power = 0.0
        overflow = False
        
    def __str__(self):
        str_info = {
            k: v
            for k, v in self.__dict__.items()
            if k not in ['domain_name', 'device', 'shunt_resistor', 'maximum_current','phase_multiplier'] and v
        }
        return str(str_info)

class PMIC(object):
    device_id = 0
    logger = None
    sensor_list = []

    def __init__(self, domains):
        self.logger = self.GetLogger()
        for d in domains:
            for r in d.rails:
                if r._device_type == DeviceType.INA226:
                    r._sensor = SensorData(r.name, d.name, INA226(r.i2c_address, r.i2c_bus), r.shunt_resistor, r.maximum_current, r.phase_multiplier)
                #elif r._device_type == DeviceType.INAxxx: # Future developments
                #    r._sensor = SensorData(r.name, d.name, INA226(r.i2c_address, r.i2c_bus), r.shunt_resistor, r.maximum_current, r.phase_multiplier)
                else:
                    self.logger.error(f"PMIC: Wrong Device/Sensor Type. railname = {r.name}")
                self.sensor_list.append(r._sensor)
        ret = self.InitBoard()
        if True != ret:
            self.logger.error(f"PMIC: InitBoardInfo failed. ret = {ret}")
        self.logger.info("Inside PMIC Constructor")
        pass

    @staticmethod
    def GetLogger():
        """
        Static method to get the logger for the class.
        Default loglevel is set inside this class

        :return: logger

        """
        log_level = logging.DEBUG
        logging.basicConfig(format="%(levelname)s:%(message)s")
        logger = logging.getLogger(__name__)
        try:
            handler_set_check = getattr(logger, 'handler_set')
        except AttributeError:
            handler_set_check = False
        if not handler_set_check:
            logger.setLevel(log_level)
            logger.handler_set = True
            logger.disabled = False
        return logger

    def InitBoard(self):
        """
        Initialize Board's sensor calibration values

        :param : None
        :return: True: Sensor initializations are successful
        """
        ret = True
        self.logger.debug("InitBoardInfo called")
        for s in self.sensor_list:
           ret &= s.device.initSensor(s.maximum_current, s.shunt_resistor, s.phase_multiplier)
        return ret

    def BoardInfo(self, eeprom):
        """
        Create Board's Info

        :param : None
        :return: Board info in json formatted
        """
        boardinfo = {}
        i2c = I2C_Client(eeprom.I2C_Bus, eeprom.I2C_Addr)
        result, eeprom_data = i2c.ReadRegister(0x00, 256)
        if result:
            offset = 0xA
            boardinfo["Language"] = eeprom_data[offset]
            #offset = 0xB
            #a = _struct_time(tm_year=96, tm_mday=1, tm_min= int.from_bytes(eeprom_data[offset:offset+2], "big"))
            #manu_time.tm_year = 96
            #manu_time.tm_mday = 1
            #manu_time.tm_min = int.from_bytes(eeprom_data[offset:offset+2], "big")
            #_time = time.mktime(a)
            #board["Manufacturing Date"] = time.ctime(_time)

            offset = 0xe
            length = int.from_bytes(eeprom_data[offset:offset+1], "big")
            length &= 0x3f
            boardinfo["Manufacturer"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')

            offset = offset + length + 1
            length = int.from_bytes(eeprom_data[offset:offset+1], "big")
            length &= 0x3f
            boardinfo["Product Name"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')

            offset = offset + length + 1
            length = int.from_bytes(eeprom_data[offset:offset+1], "big")
            length &= 0x3f
            boardinfo["Board Serial Number"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')
            
            offset = offset + length + 1
            length = int.from_bytes(eeprom_data[offset:offset+1], "big")
            length &= 0x3f
            boardinfo["Board Part Number"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')

            offset = offset + length + 1
            length = int.from_bytes(eeprom_data[offset:offset+1], "big")
            length &= 0x3f
            boardinfo["Board Revision"] = eeprom_data[(offset+1):((offset+1) + length)].decode("utf-8").strip('\x00')
        i2c.Release()
        return boardinfo

    def GetPythonLogLevels(self):
        """
        Return the logging levels supported by logging library in python

        :param : None
        :return: Dictionary showing the log levels supported by logging library
        """
        return get_python_log_levels()

    def SetServerLogLevel(self, PythonLogLevel):
        """
        Set the python log level to the given level

        :param : Log level to set
        :return: None
        """
        self.logger.debug(f"PythonLogLevel = {PythonLogLevel}")
        LogLevelsDict = get_python_log_levels()
        if PythonLogLevel == LogLevelsDict["DEBUG"]:
            self.logger.setLevel(logging.DEBUG)
        elif PythonLogLevel == LogLevelsDict["INFO"]:
            self.logger.setLevel(logging.INFO)
        elif PythonLogLevel == LogLevelsDict["WARNING"]:
            self.logger.setLevel(logging.WARNING)
        elif PythonLogLevel == LogLevelsDict["ERROR"]:
            self.logger.setLevel(logging.ERROR)
        else:
            self.logger.setLevel(logging.CRITICAL)
        return

    def GetSensor(self, domainname, railname):
        """
        Get sensor object of given domain and rail name

        :param : None
        :return: sensor object, low level sensor handler.
        """
        sensor = None
        for _s in self.sensor_list:
            if _s.domain_name == domainname and _s.name == railname:
                sensor = _s
        return sensor

    def GetSensorValues(self, sensor):
        """
        Find and Get sensor object of given domain and rail name

        :param : None
        :return: Sensor Bus Voltage, Current and Power values
        """
        sensor.current = sensor.device.getCurrent()
        sensor.vbus = sensor.device.getBusVoltage()
        sensor.power = sensor.device.getPower()
        return sensor.vbus, sensor.current, sensor.power

    def __del__(self):
        self.logger.info("Inside PMIC Destructor")



# Copyright (C) 2023 Advanced Micro Devices, Inc.  All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

__author__ = "Salih Erim"
__copyright__ = "Copyright 2023, Advanced Micro Devices, Inc."

import logging
from enum import IntEnum
from periphery import I2C


class INA226_REGS(IntEnum):
    CONFIGURATION = 0x00
    SHUNT_VOLTAGE = 0x01
    BUS_VOLTAGE = 0x02
    POWER = 0x03
    CURRENT = 0x04
    CALIBRATION = 0x05
    MASK_ENABLE = 0x06
    ALERT_LIMIT = 0x07
    MANUFACTURER = 0XFE
    DIE_ID = 0XFF


BUS_MILLIVOLTS_LSB = 1.25
SHUNT_MILLIVOLTS_LSB = 0.0025


logging.basicConfig(level=logging.ERROR)
class INA226(object):
	def __init__(self, address, devicepath):
		logging.info("Inside INA226 Constructor")
		self._address = int(address, 0)
		self._bus = I2C(devicepath)
		self._currentLSB = 0
		self._maxCurrent = 0
		self._shunt = 0
		self._phase_multiplier = 0

	def initSensor(self, mc, sr, pm):
		if 0x2260 == self.getDieID():
			if self.SetCalibration(mc, sr, pm):
				return True
			else:
				logging.error("INA226 Calibration Failed.")
				return False
		else:
			logging.error("INA226 init failed.") 
			return False


	def getAddress(self):
		return _address

	def getShuntVoltage(self):
		voltage = self._readRegister(INA226_REGS.SHUNT_VOLTAGE)
		voltage *= 2.5.e-6
		return voltage 

	def getBusVoltage(self):
		voltage = self._readRegister(INA226_REGS.BUS_VOLTAGE)
		voltage *= 1.25e-3
		return voltage
	
	def getPower(self):
		power = self._readRegister(INA226_REGS.POWER)
		power *= (self._currentLSB * 25)
		power *= self._phase_multiplier
		return power

	def getCurrent(self):
		value = self._readRegister(INA226_REGS.CURRENT)
		
		if value > 0x7fff:
			value -= 0x10000
			value = abs(value)

		current = self._currentLSB * value
		current *= self._phase_multiplier
		return current


	def SetCalibration(self, max_current, shunt_resistor, phase_multiplier):

		current_LSB = max_current / (32678 * 1000)
		calibration = (0.00512 * 1000000) / (current_LSB * shunt_resistor)

		if calibration > 32767:
			calibration = 32767

		self._writeRegister(INA226_REGS.CALIBRATION, int(calibration))

		calibration = self._readRegister(INA226_REGS.CALIBRATION)

		if calibration != 0:
			current_LSB = (0.00512 * 1000000) / (calibration * shunt_resistor)
		else:
			return False

		self._currentLSB = current_LSB
		self._maxCurrent = current_LSB * 32768
		self._shunt = shunt_resistor
		self._phase_multiplier = phase_multiplier

		return True

	
	def getManufacturerID(self):
		return self._readRegister(INA226_REGS.MANUFACTURER)

	def getDieID(self):
		return self._readRegister(int(INA226_REGS.DIE_ID))

	def _readRegister(self, reg):
		val = None
		try:
			msgs = [I2C.Message([reg]), I2C.Message([0x00, 0x00], read=True)]
			self._bus.transfer(self._address, msgs)
			logging.debug("{0}: 0x{1:02x}{2:02x}".format(self._address, msgs[1].data[0], msgs[1].data[1]))
			val = (msgs[1].data[0] << 8) + msgs[1].data[1]
		except IOError:
			logging.error("INA226 _readRegister failed.")
		return val
	

	def _writeRegister(self, reg, val):
		ret = False
		data = None
		try:
			data = bytearray(val.to_bytes(2, "big"))
			data.insert(0, reg)
			msgs = [I2C.Message(data)]
			self._bus.transfer(self._address, msgs)
			ret = True
		except IOError:
			logging.error("INA226 _writeRegister failed.")
		return ret

	def __del__(self):
		logging.info("Inside INA226 Destructor")